package com.webonlineapp.hyjavaapexdemo;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import com.hyjavaapex.chart.Apexchart;
import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.Html;
import com.vaadin.flow.component.applayout.AppLayout;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.H4;
import com.vaadin.flow.component.html.Hr;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.orderedlayout.FlexComponent.Alignment;
import com.vaadin.flow.component.popover.Popover;
import com.vaadin.flow.component.popover.PopoverPosition;
import com.vaadin.flow.component.tabs.Tab;
import com.vaadin.flow.component.textfield.TextArea;
import com.vaadin.flow.data.provider.ListDataProvider;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.webonlineapp.hyjavaapexdemo.charts.DemoChart;
import com.webonlineapp.hyjavaapexdemo.charts.DemoChartBase;
import com.webonlineapp.hyjavaapexdemo.charts.DemoCharts;
import com.webonlineapp.hyjavaapexdemo.components.ApexchartComponent;
import com.webonlineapp.hyjavaapexdemo.components.ApexchartsImageComponent;
import com.webonlineapp.hyjavaapexdemo.components.AutoPlayerComponent;
import com.webonlineapp.hyjavaapexdemo.components.JavaScriptComponent;
import com.webonlineapp.hyjavaapexdemo.components.JavaSourceComponent;
import com.webonlineapp.hyjavaapexdemo.components.PaperToggleButton;
import com.webonlineapp.hyjavaapexdemo.components.TabSheet;
import com.webonlineapp.hyjavaapexdemo.util.DateUtil;
import com.webonlineapp.hyjavaapexdemo.util.DesignerIcons;
import com.webonlineapp.hyjavaapexdemo.util.JavascriptUtil;

public class DemoPage {
	
	private Grid<DemoChart> demoChartsGrid = new Grid<DemoChart>();
	private DemoChart selectedDemoChart;
	private TabSheet tabSheet = new TabSheet();
	private VerticalLayout page1;
	private ApexchartsImageComponent apexchartsImageComponent = new ApexchartsImageComponent();
	private JavaSourceComponent page2 = new JavaSourceComponent();
	private JavaScriptComponent page3 = new JavaScriptComponent();
	
	private Component help = getHelpText();
	
	private String compHeight = "65vh";
	
	private Html jsTimeComponent = new Html("<p></p>");
	private boolean chartDisplayed = false;
	
	
	/**
	 * 
	 * @param view
	 * @param hyJavaImages
	 * @return
	 */
	public Component buildHeading(AppLayout view, boolean hyJavaImages) {
		HorizontalLayout header = new HorizontalLayout();
		header.setWidthFull();
		
        H4 title = new H4((hyJavaImages ? 
        		com.hyjavacharts.hyjavaimages.LibraryInfo.getLibraryName() +
        		" v" + com.hyjavacharts.hyjavaimages.LibraryInfo.getHyJavaImagesAPIVersion()
        		: 
        		com.hyjavaapex.LibraryInfo.getLibraryName() + 
        		" v" + com.hyjavaapex.LibraryInfo.getHyJavaApexAPIVersion())
        		+ " Library Demonstration");
        title.getStyle().set("color", "white");
        
        Span jsLibraryVersion = new Span("(with ApexCharts v" + com.hyjavaapex.LibraryInfo.getApexChartsAPIVersion() + ")");
        
        Icon options = VaadinIcon.COG_O.create();
        options.setSize("var(--lumo-icon-size-s)");
        
        Popover optionsPopover = new Popover();
        optionsPopover.setTarget(options);
        optionsPopover.setWidth("300px");
        optionsPopover.setPosition(PopoverPosition.BOTTOM_END);
        optionsPopover.setModal(true);
        
        VerticalLayout popoverLayout = new VerticalLayout();
        PaperToggleButton showMenu = new PaperToggleButton("Show Menu", true);
        showMenu.addValueChangeListener(e -> {
        	view.getElement().setAttribute("hide-menu", !e.getValue());
			JavascriptUtil.triggerResize();
		});
        
        popoverLayout.add(new H4("Demo Options"), new Hr(), showMenu, new Hr(), new AutoPlayerComponent(this, hyJavaImages));
        optionsPopover.add(popoverLayout);
        
        header.add(title);
        if (hyJavaImages) {
        	header.add(jsLibraryVersion);
        }
        header.addToEnd(options, new Span());
        header.setAlignItems(Alignment.CENTER);
        
        return header;
	}
	
	/**
	 * 
	 * @param view
	 * @param hyJavaImages
	 * @return
	 */
	public Component buildMenu(AppLayout view, boolean hyJavaImages) {
        demoChartsGrid.setHeight("100%");
        
        demoChartsGrid.addColumn(new ComponentRenderer<>(demoChart -> {
            if (demoChart.isHeading()) {
                return new Html("<span style='font-size: larger;'><b><i>" + demoChart.getName() + "</i></b>" + 
                		(demoChart.isNewChart() ? 
                		"&nbsp;&nbsp;<img src='images/new.png' width='35' height='35'>" 
                		: "") +  
                		"</span>");
            } 
            else {
                return new Html("<span>" + "&nbsp;&nbsp;&nbsp;" + demoChart.getName() + 
                		(demoChart.isNewChart() ? 
                		"&nbsp;&nbsp;<img src='images/new.png' width='25' height='25'>" 
                		: "") + 
                		"</span>");
            }
        }));
        
        demoChartsGrid.setItems(DemoCharts.getDemoChartsList());
        
        // Filter charts by display type
        ListDataProvider<DemoChart> dataProvider = (ListDataProvider<DemoChart>) demoChartsGrid.getDataProvider();
        dataProvider.setFilter((dc) -> {
    		return dc.displayChart(hyJavaImages);
    	});
        
        demoChartsGrid.addSelectionListener(event -> {
        	chartDisplayed = false;
        	Optional<DemoChart> first = event.getFirstSelectedItem();
        	
        	if (first.isPresent()) {
         		selectedDemoChart = first.get();
         		if (selectedDemoChart.getChart() != null) {
         			tabSheet.setSelectedIndex(0);

         			if (view.isOverlay()) {
         				view.setDrawerOpened(false);
         			}
         			
         			displayChart(selectedDemoChart.getChart(), selectedDemoChart.getName(), hyJavaImages);
          		}
         		
         		chartDisplayed = true;
        	}
        });
        
        if (hyJavaImages) {
        	demoChartsGrid.select(DemoCharts.getDemoChartsList().get(1));
        }
        else {
        	demoChartsGrid.select(DemoCharts.getDemoChartsList().get(0));
        }
        
        VerticalLayout menu = new VerticalLayout();
        menu.setId("the-menu");
        menu.setWidthFull();
        menu.setHeight("100%");
        menu.setSpacing(false);
        menu.setPadding(false);
        
        menu.add(demoChartsGrid);
        
        return menu;
	}
	
	/**
	 * 
	 * @param selectedDemoChart
	 * @param chartName
	 * @param hyJavaImages
	 */
	private void displayChart(DemoChartBase selectedDemoChart, String chartName, boolean hyJavaImages) {
		List<Apexchart> chartList = new ArrayList<Apexchart>();
		
		Apexchart apexchart = selectedDemoChart.configure();
		if (apexchart != null) {
			chartList.add(apexchart);
		}
		else {
			chartList = selectedDemoChart.getCharts();
		}

		String chartOptionsJs = "";
		String combinedChartOptionsJs = "";
		
		if (!hyJavaImages) {
			page1.getChildren().forEach(child -> { 
				if (child instanceof ApexchartComponent) {
					ApexchartComponent c = (ApexchartComponent) child;
					c.destroyChart();
				}
			});
		}
		page1.removeAll();
		
		if (hyJavaImages) {
			String[] chartOptionsJsArray = new String[chartList.size()];
			List<String> chartHeight = new ArrayList<String>();
			int i = 0;
			
			for (Apexchart chart : chartList) {
				chartOptionsJs = chart.chartOptionsToJs();
				combinedChartOptionsJs = combinedChartOptionsJs + chartOptionsJs;
				
				String ch = chart.getChartOptions().getChart().getHeightAsString();
				chartHeight.add(ch == null ? "100%" : ch);
				
				chartOptionsJsArray[i] = chartOptionsJs;
				i++;
			}
			
			page1.add(apexchartsImageComponent);
			
			apexchartsImageComponent.setChartHeights(chartHeight);
			apexchartsImageComponent.displayChartImage(chartOptionsJsArray);
		}
		else {
			long chartId = 0;
			LocalDateTime started = null;
			LocalDateTime ended = null;
			double totalDuration = 0;

			for (Apexchart chart : chartList) {
				started = LocalDateTime.now();
				chartOptionsJs = chart.chartOptionsToJs();
				ended  = LocalDateTime.now();
				totalDuration = totalDuration + Duration.between(started, ended).toMillis();
				combinedChartOptionsJs = combinedChartOptionsJs + chartOptionsJs;
				
				ApexchartComponent apexchartComponent = new ApexchartComponent();
				apexchartComponent.setChartId(chartId);
	 			apexchartComponent.setWidth("100%");

	 			page1.add(apexchartComponent);

	 			if (chartName.equals("Column With Group Label")) {
	 				apexchartComponent.setAdditionalJs("dayjs.extend(window.dayjs_plugin_quarterOfYear)");
	 			}
	 			else {
	 				apexchartComponent.setAdditionalJs("");
	 			}
	 			
	 			if (chartName.equals("Real Time")) {
	 				apexchartComponent.setChartId(200);
	 			}
	 			
				apexchartComponent.displayChart(chartOptionsJs);
				
				chartId++;
			}
			
			if (Properties.SHOW_CHART_OPTIONS_TO_JS_TIMING) {
				jsTimeComponent.setHtmlContent("<p style=\"font-size: small;\"><i>" + "Generating chart options using the Apexchart class chartOptionsToJs() method took " + DateUtil.formatDuration(totalDuration)  + "ms</i></p>");
			}
			page2.setJavaSource(selectedDemoChart.getClass().getName());
		}
		
        page3.setJavaScript(combinedChartOptionsJs);
 	}
	
	/**
	 * 
	 * @param content
	 * @param hyJavaImages
	 */
	public void buildTab(VerticalLayout content, boolean hyJavaImages) {
		tabSheet = new TabSheet();
		tabSheet.setWidth("100%");

		// Add tabs and pages to tabsheet
		tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), new Icon(VaadinIcon.BAR_CHART), 
				new Span(hyJavaImages ? "Chart Image" : "Example Chart"), new Span(""))), buildPage1());
		
		if (!hyJavaImages) {
			tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), DesignerIcons.COFFEE_HOT.create(), 
					new Span("Java Source Code"), new Span(""))), buildPage2());
		}
		
		tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), new Icon(VaadinIcon.CURLY_BRACKETS), 
				new Span(hyJavaImages ? "ApexCharts Chart Options" : "Generated Chart Options"), new Span(""))), buildPage3());
		
		if (Properties.SHOW_TEST_TAB) {
			tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), new Icon(VaadinIcon.CHECK), 
					new Span("Test Chart Options"), new Span(""))), buildPage4());
		}
		
		// Add all tab components to the layout
        tabSheet.addComponentsToLayout(content);
        
        tabSheet.addSelectedChangeListener(event -> {
        	if (tabSheet.getSelectedIndex() == 1) {
        		page2.prettify();
        	}
        });
	}
	
	/**
	 * 
	 * @return
	 */
	private VerticalLayout buildPage1() {
		VerticalLayout fullPage1 = new VerticalLayout();
		
		page1 = new VerticalLayout();
		page1.setHeight(compHeight);
		page1.setWidth("100%");
		page1.setMargin(false);
		page1.setPadding(false);
		page1.setSpacing(false);
		
		apexchartsImageComponent.setWidth("100%");
		apexchartsImageComponent.setHeight(compHeight);
		
		page1.add(help);
		
		fullPage1.add(page1);
		fullPage1.add(jsTimeComponent);
        
        return fullPage1;
	}
	
	/**
	 * 
	 * @return
	 */
	private JavaSourceComponent buildPage2() {
		page2.setHeight(compHeight);
		page2.setWidth("100%");
		
		return page2;
	}
	
	/**
	 * 
	 * @return
	 */
	private JavaScriptComponent buildPage3() {
		page3.setHeight(compHeight);
		page3.setWidth("100%");
		
		return page3;
	}
	
	/**
	 * 
	 * @return
	 */
	private VerticalLayout buildPage4() {
		VerticalLayout page4 = new VerticalLayout();
		
		TextArea chartOptions = new TextArea();
		chartOptions.setLabel("Chart Options");
		chartOptions.setClearButtonVisible(true);
		chartOptions.setPlaceholder("Paste chart options here...");
		chartOptions.setWidth("100%");
		chartOptions.setHeight("400px");
		
		Button displayButton = new Button("Display Chart");
		displayButton.addThemeVariants(ButtonVariant.LUMO_PRIMARY);
		
		ApexchartComponent highchartTest = new ApexchartComponent();
		highchartTest.setHeight(compHeight);
		highchartTest.setWidth("100%");
		
		page4.add(chartOptions);
		page4.add(displayButton);
		page4.add(highchartTest);

		displayButton.addClickListener(clickEvent -> {
			highchartTest.displayChart(chartOptions.getValue());
    	});

		return page4;
	}
	
	/**
	 * 
	 * @return
	 */
	private Component getHelpText() {
		HorizontalLayout hl = new HorizontalLayout();
		hl.setWidthFull();
		
		String text = 
			"<div><h4>" + com.hyjavaapex.LibraryInfo.getLibraryName() + 
			" v" + com.hyjavaapex.LibraryInfo.getHyJavaApexAPIVersion()
			+ " Library Demonstration" + "</h4>" + 
			"<p>To display an example chart, select one from the menu on the left.</p>" + 
			"<p>The Java source to configure the selected chart is shown in the <i>Java Source Code</i> tab." + 
			"<br>The ApexChart chart options generated from the library are shown in the <i>Generated Chart Options</i> tab.</p>" + 
			(Properties.SHOW_CHART_OPTIONS_TO_JS_TIMING ? "<p>The elapsed time to generate the ApexChart chart options is displayed below the chart.</p>" : "") + 
			"</div>";
		
		hl.add(new Html(text));
		
		return hl;
	}
	
	/**
	 * 
	 * @return
	 */
	public DemoChart getSelectedDemoChart() {
		return selectedDemoChart;
	}

	/**
	 * 
	 * @return
	 */
	public ApexchartsImageComponent getApexchartsImageComponent() {
		return apexchartsImageComponent;
	}

	/**
	 * 
	 * @return
	 */
	public boolean isChartDisplayed() {
		return chartDisplayed;
	}

	/**
	 * 
	 * @return
	 */
	public Grid<DemoChart> getDemoChartsGrid() {
		return demoChartsGrid;
	}

}
