package com.webonlineapp.hyjavaapexdemo.components;

import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.html.H5;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.textfield.IntegerField;
import com.vaadin.flow.component.textfield.IntegerField.IntegerFieldI18n;
import com.webonlineapp.hyjavaapexdemo.DemoPage;
import com.webonlineapp.hyjavaapexdemo.charts.DemoChart;
import com.webonlineapp.hyjavaapexdemo.charts.DemoCharts;
import com.webonlineapp.hyjavaapexdemo.util.Audio;
import com.webonlineapp.hyjavaapexdemo.util.Audio.SoundEffect;

public class AutoPlayerComponent extends VerticalLayout {
	private static final long serialVersionUID = 1L;
	
	private ScheduledExecutorService scheduledExecutorService;
	private Audio audio = new Audio();

	/**
	 * 
	 */
	public AutoPlayerComponent(DemoPage demoPage, boolean hyJavaImages) {
		this.setMargin(false);
		this.setPadding(false);
		
		IntegerField autoDelayMs = new IntegerField();
		autoDelayMs.setLabel("Delay Between Charts (ms)");
		autoDelayMs.setRequiredIndicatorVisible(true);
		autoDelayMs.setMin(1000);
		autoDelayMs.setValue(hyJavaImages ? 3000 : 2000);
		autoDelayMs.setStepButtonsVisible(true);
		autoDelayMs.setStep(250);
		autoDelayMs.setI18n(new IntegerFieldI18n()
				.setMinErrorMessage("Delay must be >= 1000")
		        .setBadInputErrorMessage("Invalid number format")
		        .setStepErrorMessage("Delay must be a multiple of 250"));
		
        PaperToggleButton backgroundAudio = new PaperToggleButton("Background Audio", true);
        Button autoPlayButton = new Button("Start");
		
		autoPlayButton.addClickListener(event -> {
			if (autoPlayButton.getText().equals("Start")) {
				if (backgroundAudio.getValue()) {
					audio.play(SoundEffect.PIANO, 0.3, true);
				}

				scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
				AutoRunCharts autoRunTask = new AutoRunCharts(UI.getCurrent(), demoPage, hyJavaImages, autoPlayButton);
				scheduledExecutorService.scheduleAtFixedRate(autoRunTask, autoDelayMs.getValue(), autoDelayMs.getValue(), TimeUnit.MILLISECONDS);
				autoPlayButton.setText("Stop");
				autoDelayMs.setEnabled(false);
			} 
			else {
				scheduledExecutorService.shutdownNow();
				audio.pause();
				autoPlayButton.setText("Start");
				autoDelayMs.setEnabled(true);
			}
		});
		
		backgroundAudio.addValueChangeListener(e -> {
			if (audio.isAudioLoaded()) {
				if (e.getValue()) {
					if (autoPlayButton.getText().equals("Stop")) {
						audio.resume();
					}
				} 
				else {
					audio.pause();
				}
			} 
			else {
				if (e.getValue()) {
					if (autoPlayButton.getText().equals("Stop")) {
						audio.play(SoundEffect.PIANO, 0.3, true);
					}
				}
			}
		});
		
		autoDelayMs.addValueChangeListener(event -> {
			if (autoDelayMs.isInvalid()) {
				autoPlayButton.setEnabled(false);
			}
			else {
				autoPlayButton.setEnabled(true);
			}
		});
		
		add(new H5("Auto Play Charts"), autoPlayButton, autoDelayMs, backgroundAudio);
	}
	
	
	
	
	/*
	 * 
	 */
	public class AutoRunCharts implements Runnable {
		private List<DemoChart> list = DemoCharts.getDemoChartsList();
		private UI ui;
		private DemoPage demoPage;
		private boolean hyJavaImages = false;
		private int row = 2;
		private Button autoPlayButton;
		
		
		/**
		 * 
		 * @param ui
		 * @param demoPage
		 */
		public AutoRunCharts(UI ui, DemoPage demoPage, boolean hyJavaImages, Button autoPlayButton) {
			this.ui = ui;
			this.demoPage = demoPage;
			this.hyJavaImages = hyJavaImages;
			this.autoPlayButton = autoPlayButton;
		}
		
		/**
		 * 
		 */
		@Override
		public void run() {
			if (row != -1 && demoPage.isChartDisplayed()) {
				ui.access(() -> { 
					row = findNextChart();
					
					demoPage.getDemoChartsGrid().select(list.get(row));
					demoPage.getDemoChartsGrid().scrollToItem(list.get(findPrevChart(row)));
					
					if (hyJavaImages) {
						demoPage.getApexchartsImageComponent().clickGenerate();
					}
					
					row++;
					if (row >= list.size()) {
						row = -1; 	// stop
					}
				});
			}
			
			if (row == -1) {
				ui.access(() -> { 
					autoPlayButton.click();
				});
			}
		}
		
		/**
		 * 
		 * @return
		 */
		private int findNextChart() {
			int row = 0;
			
			// look for selected chart
			for (DemoChart item : list) {
				if (item == demoPage.getSelectedDemoChart()) {
					break;
				}
				
				row++;
				
				if (row >= list.size()) {
					row = 0;
				}
			}
			
			// next one
			row++;
			if (row >= list.size()) {
				row = 0;
			}
			
			// look for valid next chart
			while (true) {
				DemoChart item = list.get(row);
				
				if (item.isHeading()) {
					row++;
				}
				
				if (item.displayChart(hyJavaImages)) {
					return row;
				}
				
				row++;
				
				if (row >= list.size()) {
					return 1;
				}
			}
		}
		
		/**
		 * Want to scroll to current row but back 5 rows.
		 * @return
		 */
		private int findPrevChart(int currRow) {
			int newRow = currRow;
			
			int found = 0;
			newRow--;
			
			while (true) {
				DemoChart item = list.get(newRow);
				
				if (item.isHeading() || item.displayChart(hyJavaImages)) {
					found++;
				}
				
				if (found == 5) {
					break;
				}
				
				newRow--;
				
				if (newRow < 0) {
					newRow = 2;
					break;
				}
			}
			
			return newRow;
		}
	}

}
