package com.webonlineapp.hyjavastockdemo12;

import java.time.LocalDateTime;
import java.util.Optional;

import com.hyjavastock.chart.Highstock;
import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.Html;
import com.vaadin.flow.component.applayout.AppLayout;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.H4;
import com.vaadin.flow.component.html.Hr;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.orderedlayout.FlexComponent.Alignment;
import com.vaadin.flow.component.popover.Popover;
import com.vaadin.flow.component.popover.PopoverPosition;
import com.vaadin.flow.component.tabs.Tab;
import com.vaadin.flow.component.textfield.TextArea;
import com.vaadin.flow.data.provider.ListDataProvider;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.webonlineapp.hyjavastockdemo12.charts.DemoChart;
import com.webonlineapp.hyjavastockdemo12.charts.DemoCharts;
import com.webonlineapp.hyjavastockdemo12.components.AutoPlayerComponent;
import com.webonlineapp.hyjavastockdemo12.components.HighstockComponent;
import com.webonlineapp.hyjavastockdemo12.components.HighstockImageComponent;
import com.webonlineapp.hyjavastockdemo12.components.JavaScriptComponent;
import com.webonlineapp.hyjavastockdemo12.components.JavaSourceComponent;
import com.webonlineapp.hyjavastockdemo12.components.PaperToggleButton;
import com.webonlineapp.hyjavastockdemo12.components.TabSheet;
import com.webonlineapp.hyjavastockdemo12.util.DateUtil;
import com.webonlineapp.hyjavastockdemo12.util.DesignerIcons;
import com.webonlineapp.hyjavastockdemo12.util.JavascriptUtil;

public class DemoPage {
	
	private Grid<DemoChart> demoChartsGrid = new Grid<DemoChart>();
	private DemoChart selectedDemoChart;
	private TabSheet tabSheet = new TabSheet();
	private HighstockComponent highchartComponent = new HighstockComponent();
	private HighstockImageComponent highchartImageComponent = new HighstockImageComponent();
	private JavaSourceComponent page2 = new JavaSourceComponent();
	private JavaScriptComponent page3 = new JavaScriptComponent();
	private String prevChart = "";
	
	private Component help = getHelpText();
	private Button playButton = new Button("Play Chart");
	
	private String compHeight = "65vh";
	
	private Html jsTimeComponent = new Html("<p></p>");
	private boolean chartDisplayed = false;
	
	
	/**
	 * 
	 * @param view
	 * @param hyJavaImages
	 * @return
	 */
	public Component buildHeading(AppLayout view, boolean hyJavaImages) {
		HorizontalLayout header = new HorizontalLayout();
		header.setWidthFull();
		
		H4 title = new H4((hyJavaImages ? 
	    		com.hyjavacharts.hyjavaimages.LibraryInfo.getLibraryName() +
	    		" v" + com.hyjavacharts.hyjavaimages.LibraryInfo.getHyJavaImagesAPIVersion()
	    		: 
	    		com.hyjavastock.LibraryInfo.getLibraryName() + 
	    		" v" + com.hyjavastock.LibraryInfo.getHyJavaStockAPIVersion())
	    		+ " Library Demonstration");
        title.getStyle().set("color", "white");
        
        Span jsLibraryVersion = new Span("(with Highcharts Stock v" + com.hyjavastock.LibraryInfo.getHighstockAPIVersion() + ")");
        
        Icon options = VaadinIcon.COG_O.create();
        options.setSize("var(--lumo-icon-size-s)");
        
        Popover optionsPopover = new Popover();
        optionsPopover.setTarget(options);
        optionsPopover.setWidth("300px");
        optionsPopover.setPosition(PopoverPosition.BOTTOM_END);
        optionsPopover.setModal(true);
        
        VerticalLayout popoverLayout = new VerticalLayout();
        PaperToggleButton showMenu = new PaperToggleButton("Show Menu", true);
        showMenu.addValueChangeListener(e -> {
        	view.getElement().setAttribute("hide-menu", !e.getValue());
			JavascriptUtil.triggerResize();
		});
        
        popoverLayout.add(new H4("Demo Options"), new Hr(), showMenu, new Hr(), new AutoPlayerComponent(this, hyJavaImages));
        optionsPopover.add(popoverLayout);
        
        header.add(title);
        if (hyJavaImages) {
        	header.add(jsLibraryVersion);
        }
        header.addToEnd(options, new Span());
        header.setAlignItems(Alignment.CENTER);
        
        return header;
	}
	
	/**
	 * 
	 * @param view
	 * @param hyJavaImages
	 * @return
	 */
	public Component buildMenu(AppLayout view, boolean hyJavaImages) {
        demoChartsGrid.setHeight("100%");
        
        demoChartsGrid.addColumn(new ComponentRenderer<>(demoChart -> {
        	if (demoChart.isHeading()) {
                return new Html("<span style='font-size: larger;'><b><i>" + demoChart.getName() + "</i></b>" + 
                		(demoChart.isNewChart() ? 
                		"&nbsp;&nbsp;<img src='images/new.png' width='35' height='35'>" 
                		: "") +  
                		"</span>");
            } 
            else {
                return new Html("<span>" + "&nbsp;&nbsp;&nbsp;" + demoChart.getName() + 
                		(demoChart.isNewChart() ? 
                		"&nbsp;&nbsp;<img src='images/new.png' width='25' height='25'>" 
                		: "") + 
                		"</span>");
            }
        }));
        
        demoChartsGrid.setItems(DemoCharts.getDemoChartsList());
        
        // Filter charts by display type
        ListDataProvider<DemoChart> dataProvider = (ListDataProvider<DemoChart>) demoChartsGrid.getDataProvider();
        dataProvider.setFilter((dc) -> {
    		return dc.displayChart(hyJavaImages);
    	});
        
        demoChartsGrid.addSelectionListener(event -> {
        	chartDisplayed = false;
        	Optional<DemoChart> first = event.getFirstSelectedItem();
        	
        	if (first.isPresent()) {
        		selectedDemoChart = first.get();
        		
         		if (selectedDemoChart.getChart() != null) {
         			help.setVisible(false);
         			highchartComponent.setVisible(true);
         			jsTimeComponent.setVisible(true);
         			tabSheet.setSelectedIndex(0);

         			if (view.isOverlay()) {
         				view.setDrawerOpened(false);
         			}
         			
         			displayChart(selectedDemoChart, selectedDemoChart.getName(), hyJavaImages);
          		}
         		
         		chartDisplayed = true;
        	}
        });
        
        if (hyJavaImages) {
        	demoChartsGrid.select(DemoCharts.getDemoChartsList().get(1));
        }
        else {
        	demoChartsGrid.select(DemoCharts.getDemoChartsList().get(0));
        }
        
        VerticalLayout menu = new VerticalLayout();
        menu.setId("the-menu");
        menu.setWidthFull();
        menu.setHeight("100%");
        menu.setSpacing(false);
        menu.setPadding(false);
        
        menu.add(demoChartsGrid);
        
        return menu;
	}
	
	/**
	 * 
	 * @param selectedDemoChart
	 * @param chartName
	 * @param hyJavaImages
	 */
	private void displayChart(DemoChart selectedDemoChart, String chartName, boolean hyJavaImages) {
		Highstock hc = selectedDemoChart.getChart().configure();
		String globalOptionsJs = hc.globalOptionsToJs();
		LocalDateTime started = LocalDateTime.now();
		String chartOptionsJs = hc.chartOptionsToJs();
		String fetchUrl = selectedDemoChart.getChart().getFetchUrl();
		LocalDateTime ended = LocalDateTime.now();
		
		if (hyJavaImages) {
			highchartImageComponent.displayChartImage(globalOptionsJs, chartOptionsJs, selectedDemoChart.getChart().isHighcharts());
		}
		else {
			highchartComponent.setHighcharts(selectedDemoChart.getChart().isHighcharts());
			
			if (selectedDemoChart.getChart().getHeight() == null) {
				highchartComponent.setHeight(compHeight);
			}
			else {
				highchartComponent.setHeight(selectedDemoChart.getChart().getHeight());
			}
			
			highchartComponent.displayChart(globalOptionsJs, chartOptionsJs, fetchUrl);
			if (Properties.SHOW_CHART_OPTIONS_TO_JS_TIMING) {
				jsTimeComponent.setHtmlContent("<p style=\"font-size: small;\"><i>" + "Generating chart options using the Highstock class chartOptionsToJs() method took " + DateUtil.formatDuration(started, ended)  + "ms</i></p>");
			}

			if (selectedDemoChart.getChart().isAudio()) {
				playButton.setVisible(true);
			}
			else {
				playButton.setVisible(false);
			}	
			
			if (prevChart.contains("Parallel Coordinates")) {
				JavascriptUtil.removeStyleSheetRule(0);
			}
			
			if (selectedDemoChart.getName().contains("Parallel Coordinates")) {
				JavascriptUtil.addStyleSheetRule(JavascriptUtil.HIGHCHARTS_SERIES_HOVER, 0);
			}
			
			prevChart = selectedDemoChart.getName();
			page2.setJavaSource(selectedDemoChart.getChart().getClass().getName());
		}
        
        page3.setJavaScript(chartOptionsJs);
	}
	
	/**
	 * 
	 * @param content
	 * @param hyJavaImages
	 */
	public void buildTab(VerticalLayout content, boolean hyJavaImages) {
		tabSheet = new TabSheet();
		tabSheet.setWidth("100%");

		// Add tabs and pages to tabsheet
		tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), new Icon(VaadinIcon.BAR_CHART), 
				new Span(hyJavaImages ? "Chart Image" : "Example Chart"), new Span(""))), buildPage1(hyJavaImages));
		
		if (!hyJavaImages) {
			tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), DesignerIcons.COFFEE_HOT.create(), 
					new Span("Java Source Code"), new Span(""))), buildPage2());
		}
		
		tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), new Icon(VaadinIcon.CURLY_BRACKETS), 
				new Span(hyJavaImages ? "Highcharts Stock Chart Options" : "Generated Chart Options"), new Span(""))), buildPage3());
		
		if (Properties.SHOW_TEST_TAB) {
			tabSheet.addTab(new Tab(new HorizontalLayout(new Span(""), new Icon(VaadinIcon.CHECK), 
					new Span("Test Chart Options"), new Span(""))), buildPage4());
		}
		
		// Add all tab components to the layout
        tabSheet.addComponentsToLayout(content);
        
        tabSheet.addSelectedChangeListener(event -> {
        	if (tabSheet.getSelectedIndex() == 1) {
        		page2.prettify();
        	}
        });
	}
	
	/**
	 * 
	 * @param hyJavaImages
	 * @return
	 */
	private VerticalLayout buildPage1(boolean hyJavaImages) {
		VerticalLayout fullPage1 = new VerticalLayout();
		
		HorizontalLayout page1 = new HorizontalLayout();
		page1.setWidth("100%");
		page1.setMargin(false);
        
        highchartComponent.setWidth("100%");
        highchartComponent.setHeight(compHeight);
        highchartImageComponent.setWidth("100%");
        highchartImageComponent.setHeight(compHeight);
        
        if (hyJavaImages) {
        	page1.add(highchartImageComponent);
        }
        else {
        	page1.add(help);
        	page1.add(highchartComponent);
        	highchartComponent.setVisible(false);
        	jsTimeComponent.setVisible(false);
        }

        fullPage1.add(page1);
        fullPage1.add(jsTimeComponent);
        
        if (!hyJavaImages) {
            playButton.addThemeVariants(ButtonVariant.LUMO_PRIMARY);

    		playButton.addClickListener(clickEvent -> {
    			highchartComponent.play();
    		});
    		
    		fullPage1.add(playButton);
    		playButton.setVisible(false);
        }
        
        return fullPage1;
	}
	
	/**
	 * 
	 * @return
	 */
	private JavaSourceComponent buildPage2() {
		page2.setHeight(compHeight);
		page2.setWidth("100%");
		
		return page2;
	}
	
	/**
	 * 
	 * @return
	 */
	private JavaScriptComponent buildPage3() {
		page3.setHeight(compHeight);
		page3.setWidth("100%");
		
		return page3;
	}
	
	/**
	 * 
	 * @return
	 */
	private VerticalLayout buildPage4() {
		VerticalLayout page4 = new VerticalLayout();
		
		TextArea chartOptions = new TextArea();
		chartOptions.setLabel("Chart Options");
		chartOptions.setClearButtonVisible(true);
		chartOptions.setPlaceholder("Paste chart options here...");
		chartOptions.setWidth("100%");
		chartOptions.setHeight("400px");
		
		Button displayButton = new Button("Display Chart");
		displayButton.addThemeVariants(ButtonVariant.LUMO_PRIMARY);
		
		HighstockComponent highchartTest = new HighstockComponent();
		highchartTest.setHeight(compHeight);
		highchartTest.setWidth("100%");
		
		page4.add(chartOptions);
		page4.add(displayButton);
		page4.add(highchartTest);

		displayButton.addClickListener(clickEvent -> {
			highchartTest.displayChart(chartOptions.getValue());
    	});

		return page4;
	}
	
	/**
	 * 
	 * @return
	 */
	private Component getHelpText() {
		HorizontalLayout hl = new HorizontalLayout();
		hl.setWidthFull();
		
		String text = 
			"<div><h4>" + com.hyjavastock.LibraryInfo.getLibraryName() + 
			" v" + com.hyjavastock.LibraryInfo.getHyJavaStockAPIVersion()
			+ " Library Demonstration" + "</h4>" + 
			"<p>To display an example chart, select one from the menu on the left.</p>" + 
			"<p>The Java source to configure the selected chart is shown in the <i>Java Source Code</i> tab." + 
			"<br>The Highchart Stock chart options generated from the library are shown in the <i>Generated Chart Options</i> tab.</p>" + 
			(Properties.SHOW_CHART_OPTIONS_TO_JS_TIMING ? "<p>The elapsed time to generate the Highchart Stock chart options is displayed below the chart.</p></div>" : "");
		
		hl.add(new Html(text));
		
		return hl;
	}
	
	/**
	 * 
	 * @return
	 */
	public DemoChart getSelectedDemoChart() {
		return selectedDemoChart;
	}

	/**
	 * 
	 * @return
	 */
	public HighstockImageComponent getHighchartImageComponent() {
		return highchartImageComponent;
	}

	/**
	 * 
	 * @return
	 */
	public boolean isChartDisplayed() {
		return chartDisplayed;
	}

	/**
	 * 
	 * @return
	 */
	public Grid<DemoChart> getDemoChartsGrid() {
		return demoChartsGrid;
	}

}
