package com.webonlineapp.hyjavastockdemo12.examplecharts;

import com.hyjavastock.chart.Highstock;
import com.hyjavastock.model.common.Function;
import com.hyjavastock.model.highstock.ChartOptions;
import com.hyjavastock.model.highstock.YAxis;
import com.hyjavastock.model.highstock.constants.HorizontalAlign;
import com.hyjavastock.model.highstock.constants.SeriesType;
import com.hyjavastock.model.highstock.constants.ToolTipShape;
import com.hyjavastock.model.highstock.series.SeriesColumn;
import com.hyjavastock.model.highstock.series.SeriesOhlc;
import com.webonlineapp.hyjavastockdemo12.chartdata.DemoChartData;
import com.webonlineapp.hyjavastockdemo12.charts.DemoChartBase;

public class StockChartWithGUI extends DemoChartBase {
	private static final long serialVersionUID = 1L;

	public Highstock configure() {
		Highstock highStock = new Highstock();
		ChartOptions chartOptions = highStock.getChartOptions();
		
		chartOptions.getExporting().setEnabled(false);
		
		YAxis yAxi = new YAxis();
		yAxi.setHeight("80%").getLabels().setAlign(HorizontalAlign.LEFT);
		yAxi.getResize().setEnabled(true);
		chartOptions.getYAxis().add(yAxi);
		
		yAxi = new YAxis();
		yAxi.setTop("80%").setHeight("20%").setOffset(0).getLabels().setAlign(HorizontalAlign.LEFT);
		chartOptions.getYAxis().add(yAxi);
		
		chartOptions.getRangeSelector().setSelected(4);
		
		String positionerJs = "var chart = this.chart, position; if (point.isHeader) { position = { x: Math.max( chart.plotLeft, Math.min( point.plotX + chart.plotLeft - width / 2,  chart.chartWidth - width - chart.marginRight) ), y: point.plotY }; } else { position = { x: point.series.chart.plotLeft, y: point.series.yAxis.top - chart.plotTop }; } return position;";
		Function positioner = new Function(positionerJs);
		positioner.addParameter("width");
		positioner.addParameter("height");
		positioner.addParameter("point");

		chartOptions.getTooltip()
			.setShape(ToolTipShape.RECT)
			.setHeaderShape(ToolTipShape.CALLOUT)
			.setBorderWidth(0)
			.setShadow(false)
			.setPositioner(positioner);
		
		SeriesOhlc seriesOhlc = new SeriesOhlc();
		seriesOhlc.setType(SeriesType.CANDLESTICK).setId("nvidia-candlestick").setName("NVIDIA Corp Stock Price");
		seriesOhlc.getDataAsJsValue().setJsValue(DemoChartData.getResource("NVIDIA-ohlc.json"));
		seriesOhlc.getDataGrouping().setGroupPixelWidth(20);
		chartOptions.getSeries().add(seriesOhlc);
		
		SeriesColumn seriesColumn = new SeriesColumn();
		seriesColumn.setType(SeriesType.COLUMN).setId("nvidia-volume").setName("NVIDIA Volume").setYAxis(1);
		seriesColumn.getDataAsJsValue().setJsValue(DemoChartData.getResource("NVIDIA-v.json"));
		chartOptions.getSeries().add(seriesColumn);
		
		return highStock;
	}
	
}